local myMODULES = merGuildResearchNotes.MODULES
local Settings = myMODULES.Settings

local Main = myMODULES.Main
local SavedVars = myMODULES.SavedVars
local Utils = myMODULES.Utils
local LAM = LibStub:GetLibrary("LibAddonMenu-2.0")


local function getEmptyGuildName(guildIndex)
    local guildName = zo_strformat(SI_EMPTY_GUILD_CHANNEL_NAME, guildIndex)
    return "<" .. guildName .. ">"
end


local function merColorize(value)
    return "|ce5ca5c" .. tostring(value) .. "|r"
end


local function mkoptGuildCheckbox(guildIndex)
    return {
        type = "checkbox",
        name = getEmptyGuildName(guildIndex),

        getFunc = function()
            local guildName = GetGuildName(GetGuildId(guildIndex))
            local enabled = SavedVars.get("guildSettings", guildName, "enabled")
            return enabled and true or false
        end,

        setFunc = function(value)
            local guildId = GetGuildId(guildIndex)
            local guildName = GetGuildName(guildId)
            if guildName ~= "" then
                local guildSettings = SavedVars.sub("guildSettings", guildName)
                if value then
                    SavedVars.lastRefresh = 0
                end
                if SavedVars.DEBUG then
                    SavedVars.aggregateGRN = nil
                end
                guildSettings.enabled = value
                Main.updatePlayerGRN(guildIndex)
            end
        end,

        disabled = function()
            local guildId = GetGuildId(guildIndex)
            return not guildId or guildId == 0
        end,

        onShow = function(control)
            if control.label then
                local guildId = GetGuildId(guildIndex)
                local guildName = GetGuildName(guildId)
                if guildName == "" then
                    guildName = getEmptyGuildName(guildIndex)
                end
                control.label:SetText(guildName)
            end
        end,
    }
end


local function panelShown(panel)
    -- controls are created when the panel is shown for the first time,
    -- before this post-handler is called;
    -- now that controlsToRefresh has been filled, disable the option
    -- to fire refresh callbacks on every update
    panel.data.registerForRefresh = false

    for i, control in ipairs(panel.controlsToRefresh) do
        if control.data.onShow then
            control.data.onShow(control)
        end
    end
end


function Settings.init()
    local panelData =
    {
        type = "panel",
        name = "Mer Guild Research Notes",
        displayName = merColorize("Mer") .. " Guild Research Notes",
        author = merColorize(merGuildResearchNotes.AUTHOR),
        version = merColorize(merGuildResearchNotes.VERSION),
        registerForRefresh = true, -- only to have controlsToRefresh filled
    }
    local optionsData = setmetatable({}, {__index = table})

    Settings.name = "merGuildResearchNotes_Settings"
    Settings.panel = LAM:RegisterAddonPanel(Settings.name, panelData)
    Utils.postHookHandler(Settings.panel, "OnShow", panelShown)

    if Settings.panel.info then
        Settings.panel.info:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end

    for guildIndex = 1, MAX_GUILDS do
        optionsData:insert(mkoptGuildCheckbox(guildIndex))
    end

    LAM:RegisterOptionControls(Settings.name, optionsData)
end

