local myMODULES = merGuildResearchNotes.MODULES
local Utils = myMODULES.Utils

local floor = math.floor
local strformat = string.format


local function tostrings(...)
    if select("#", ...) > 0 then
        return tostring(...), tostrings(select(2, ...))
    end
end


local function addMessageToBuffer(buffer, format, ...)
    local status, message = pcall(strformat, format, tostrings(...))
    if not status then
        if type(format) == "string" then
            format = strformat("%q", format)
        else
            format = tostring(format)
        end
        local argline = "|cff1919error formatting|r " .. format
        message = argline .. "\n|cff1919" .. message
    end
    local lg = 0.686275 -- INTERFACE_TEXT_COLOR_DEFAULT_TEXT
    buffer:AddMessage(message, lg, lg, lg, CHAT_CATEGORY_SYSTEM)
end


local function df(format, ...)
    local container = CHAT_SYSTEM and CHAT_SYSTEM.primaryContainer
    if container and container.currentBuffer then
        addMessageToBuffer(container.currentBuffer, format, ...)
        container:SyncScrollToBuffer()
    end
end


Utils.df = df


function Utils.pwrap(fun)
    local function success_results(status, ...)
        if status then
            return ...
        else
            df("|cff1919%s", ...)
        end
    end
    local function pcall_wrapper(...)
        return success_results(pcall(fun, ...))
    end
    return pcall_wrapper
end


function Utils.clearbit(x, n)
    local bit = 2 ^ n
    if x % (2 * bit) < bit then
        return x
    else
        return x - bit
    end
end


function Utils.countsetbits(x)
    local n = 0
    while x > 0 do
        n = n + (x % 2)
        x = floor(x / 2)
    end
    return n
end


function Utils.getbit(x, n)
    local bit = 2 ^ n
    if x % (2 * bit) < bit then
        return 0
    else
        return 1
    end
end


function Utils.setbit(x, n)
    local bit = 2 ^ n
    if x % (2 * bit) < bit then
        return x + bit
    else
        return x
    end
end


local function splitbits(x, n, ...)
    if select("#", ...) > 0 then
        local m = select(1, ...)
        local y = floor((x % (2 ^ n)) / (2 ^ (n - m)))
        return y, splitbits(x, n - m, select(2, ...))
    end
end
Utils.splitbits = splitbits


function Utils.testbit(x, n)
    local bit = 2 ^ n
    if x % (2 * bit) < bit then
        return false
    else
        return true
    end
end


function Utils.gettable(tab, ...)
    for i = 1, select("#", ...) do
        local key = select(i, ...)
        tab = tab[key]
        if type(tab) ~= "table" then
            return nil
        end
    end
    return tab
end


function Utils.getvalue(tab, ...)
    for i = 1, select("#", ...) do
        local typ = type(tab)
        if typ == "table" or typ == "userdata" then
            local key = select(i, ...)
            tab = tab[key]
        else
            return nil
        end
    end
    return tab
end


function Utils.subtable(tab, ...)
    for i = 1, select("#", ...) do
        local key = select(i, ...)
        local sub = tab[key]
        if type(sub) ~= "table" then
            sub = {}
            tab[key] = sub
        end
        tab = sub
    end
    return tab
end


--[[
    An alternative to `ipairs` for tables with holes in them.
    Unlike `ipairs`, which throws an error if (tab == nil), this
    function simply doesn't generate any pairs in such case.

    Use like this:

        for k, v in iterExisting(tab, i, j) do
            print(k, v)
        end

    which is equivalent to:

        if tab ~= nil then
            for k = i, j do
                local v = tab[k]
                if v ~= nil then
                    print(k, v)
                end
            end
        end
--]]
function Utils.iterExisting(tab, i, j)
    local function inext(s, k)
        while k < s do
            k = k + 1
            local val = tab[k]
            if val ~= nil then
                return k, val
            end
        end
    end
    if tab ~= nil then
        return inext, j, i - 1
    else
        return inext, 0, 0
    end
end


function Utils.iterSmithingResearchLineTraits(craftingType, lineIndex)
    local function nextTrait(numTraits, traitIndex)
        if traitIndex < numTraits then
            traitIndex = traitIndex + 1
            local traitType, _, known = GetSmithingResearchLineTraitInfo(
                                        craftingType, lineIndex, traitIndex)
            local traitName = GetString("SI_ITEMTRAITTYPE", traitType)
            return traitIndex, traitType, traitName, known
        end
    end
    local _, _, numTraits = GetSmithingResearchLineInfo(craftingType, lineIndex)
    return nextTrait, numTraits, 0
end


function Utils.postHook(object, funcName, hookFunc)
    local existingFunc = object[funcName]
    if type(existingFunc) == "function" then
        local function newFunc(...)
            existingFunc(...)
            hookFunc(...)
        end
        object[funcName] = newFunc
    end
end


function Utils.postHookHandler(control, handlerName, hookFunc)
    local existingHandlerFunc = control:GetHandler(handlerName)
    if existingHandlerFunc then
        local function newHandlerFunc(...)
            existingHandlerFunc(...)
            hookFunc(...)
        end
        control:SetHandler(handlerName, newHandlerFunc)
    else
        control:SetHandler(handlerName, hookFunc)
    end
end


local function timeItHelper(msg, t1, ...)
    local t2 = GetGameTimeMilliseconds()
    df("%s took %sms", msg, t2 - t1)
    return ...
end


function Utils.timeIt(obj, funcName, ...)
    local msg = (obj.DEBUG_PREFIX or "") .. funcName
    local t1 = GetGameTimeMilliseconds()
    return timeItHelper(msg, t1, obj[funcName](...))
end

